@@ -50,13 +50,52 @@
 #include "auth.h"
 #include "misc.h"
 
+#include "crypto.h"
+
+#if SSLEAY_VERSION_NUMBER >= 0x0800
+#define SSLEAY8
+#endif
+ 
+#ifdef SSLEAY8
+#define ONELINE_NAME(X) X509_NAME_oneline(X,NULL,0)
+#else
+#define ONELINE_NAME(X) X509_NAME_oneline(X)
+#endif
+
+#ifdef SSLEAY8
+#include "bio.h"
+#endif
+#include "buffer.h"
+
 #include "x509.h"
 #include "ssl.h"
 
+/* quick translation ... */
+#ifndef VERIFY_ERR_UNABLE_TO_GET_ISSUER
+#define VERIFY_ERR_UNABLE_TO_GET_ISSUER X509_V_ERR_UNABLE_TO_GET_ISSUER_CERT
+#endif
+#ifndef VERIFY_ERR_DEPTH_ZERO_SELF_SIGNED_CERT
+#define VERIFY_ERR_DEPTH_ZERO_SELF_SIGNED_CERT X509_V_ERR_DEPTH_ZERO_SELF_SIGNED_CERT 
+#endif
+#ifndef VERIFY_OK
+#define VERIFY_OK X509_V_OK
+#endif
+#ifndef VERIFY_ERR_UNABLE_TO_GET_ISSUER
+#define VERIFY_ERR_UNABLE_TO_GET_ISSUER X509_V_ERR_UNABLE_TO_GET_ISSUER_CERT
+#endif
+
+/* need to think about this mapping in terms of what the real
+ * equivalent of this actually is
+ */
+#ifndef VERIFY_ROOT_OK
+#define VERIFY_ROOT_OK VERIFY_OK
+#endif
+
 extern int auth_debug_mode;
 static auth_ssl_valid = 0;
 static char *auth_ssl_name = 0;    /* this holds the oneline name */
 
+extern BIO *bio_err;
 extern int ssl_only_flag;
 extern int ssl_debug_flag;
 extern int ssl_active_flag;
@@ -114,10 +153,10 @@ int verbose;
 	if (peer != NULL) {
 	    char *str;
 
-	    str=X509_NAME_oneline(X509_get_subject_name(peer));
+	    str=ONELINE_NAME(X509_get_subject_name(peer));
 	    fprintf(stderr,"[SSL subject=%s]\r\n",str);
 	    free(str);
-	    str=X509_NAME_oneline(X509_get_issuer_name(peer));
+	    str=ONELINE_NAME(X509_get_issuer_name(peer));
 	    fprintf(stderr,"[SSL issuer=%s]\r\n",str);
 	    free(str);
 	    X509_free(peer);
@@ -497,49 +536,31 @@ int cnt, buflen;
 	}
 }
 
-static int
-old_verify_callback(ok, xs, xi, depth, error)
-int ok;
-char *xs, *xi;
-int depth, error;
-{
-    /* ONLY WHEN DEBUGGING DO WE ALLOW SELF SIGNED CLIENT 
-     * CERTIFICATES AS SELF SIGNED CLIENT CERTIFICATES ARE POINTLESS!
-     */
-    if (ssl_debug_flag) {
-	if (error==VERIFY_ERR_DEPTH_ZERO_SELF_SIGNED_CERT)
-	  return 1;
-    }
-
-    /*
-     * If the verification fails, then don't remember the name.  However,
-     * if we don't require a certificate, then return success which will
-     * still allow us to set up an encrypted session.
-     *
-     */
-    if (!ok) {
-	/* If we can't verify the issuer, then don't accept the name. */
-	if (depth != 0 && auth_ssl_name) {
-		free(auth_ssl_name);
-		auth_ssl_name = 0;
-	}
-	return 0;
-    }
-    if (depth == 0)
-	auth_ssl_name =
-	    (char *)X509_NAME_oneline(X509_get_subject_name((X509 *)xs));
-    return ok;
-}
 
 int
+#ifdef SSLEAY8
+server_verify_callback(ok, ctx)
+int ok;
+X509_STORE_CTX *ctx;
+#else /* !SSLEAY8 */
 server_verify_callback(ok, xs, xi, depth, error)
 int ok;
 char *xs, *xi;
 int depth, error;
+#endif /* SSLEAY8 */
 {
     static char *saved_subject=NULL;
     X509 *peer;
     char *subject, *issuer;
+#ifdef SSLEAY8
+    int depth,error;
+    char *xs;
+
+    depth=ctx->error_depth;
+    error=ctx->error;
+    xs=(char *)X509_STORE_CTX_get_current_cert(ctx);
+
+#endif /* SSLEAY8 */
 
 #ifdef LOCAL_DEBUG
     if (ssl_debug_flag) {
@@ -555,14 +576,14 @@ int depth, error;
      * certificate that is being verified ... and if we cannot
      * determine that then something is seriously wrong!
      */
-    subject=(char *)X509_NAME_oneline(X509_get_subject_name((X509 *)xs));
+    subject=(char *)ONELINE_NAME(X509_get_subject_name((X509 *)xs));
     if (subject==NULL) {
 	if (ssl_debug_flag) 
 	    ERR_print_errors(bio_err);
 	ok=0;
 	goto return_time;
     }
-    issuer=(char *)X509_NAME_oneline(X509_get_issuer_name((X509 *)xs));
+    issuer=(char *)ONELINE_NAME(X509_get_issuer_name((X509 *)xs));
     if (issuer==NULL) {
 	if (ssl_debug_flag)
 	    ERR_print_errors(bio_err);
@@ -665,13 +686,28 @@ return_time: ;
 }
 
 int
+#ifdef SSLEAY8
+client_verify_callback(ok, ctx)
+int ok;
+X509_STORE_CTX *ctx;
+#else /* !SSLEAY8 */
 client_verify_callback(ok, xs, xi, depth, error)
 int ok;
 char *xs, *xi;
 int depth, error;
+#endif /* SSLEAY8 */
 {
     X509 *peer;
     char *subject, *issuer;
+#ifdef SSLEAY8
+    int depth,error;
+    char *xs;
+
+    depth=ctx->error_depth;
+    error=ctx->error;
+    xs=(char *)X509_STORE_CTX_get_current_cert(ctx);
+
+#endif /* SSLEAY8 */
 
 #ifdef LOCAL_DEBUG
     fprintf(stderr,"ssl:client_verify_callback:depth=%d ok=%d err=%d-%s\n",
@@ -685,13 +721,13 @@ int depth, error;
      * certificate that is being verified ... and if we cannot
      * determine that then something is seriously wrong!
      */
-    subject=(char *)X509_NAME_oneline(X509_get_subject_name((X509 *)xs));
+    subject=(char *)ONELINE_NAME(X509_get_subject_name((X509 *)xs));
     if (subject==NULL) {
 	ERR_print_errors(bio_err);
 	ok=0;
 	goto return_time;
     }
-    issuer=(char *)X509_NAME_oneline(X509_get_issuer_name((X509 *)xs));
+    issuer=(char *)ONELINE_NAME(X509_get_issuer_name((X509 *)xs));
     if (issuer==NULL) {
 	ERR_print_errors(bio_err);
 	ok=0;
