@@ -79,49 +79,6 @@ static char sccsid[] = "@(#)commands.c	5.5 (Berkeley) 3/22/91";
 #include <netinet/ip.h>
 #endif /* LINUX */
 
-
-#ifdef USE_SSL
-
-SSL_CTX *ssl_ctx;
-SSL *ssl_con;
-int ssl_debug_flag=0;
-int ssl_only_flag=0;
-int ssl_active_flag=0;
-int ssl_verify_flag=SSL_VERIFY_NONE;
-int ssl_secure_flag=0;
-int ssl_certsok_flag=1;
-int ssl_cert_required=0;
-int ssl_verbose_flag=0;
-int ssl_disabled_flag=0;
-char *cert_file=NULL;
-char *key_file=NULL;
-char *cipher_list=NULL;
-
-extern int display_connect_details();
-extern int server_verify_callback();
-extern int client_verify_callback();
-
-static void client_info_callback(s,where,ret)
-SSL *s;
-int where;
-int ret;
-{
-	if (where == SSL_CB_CONNECT_LOOP) {
-		fprintf(stderr,"SSL_connect:%s %s\n",SSL_state_string(s),SSL_state_string_long(s));
-	} else if (where == SSL_CB_CONNECT_EXIT) {
-		if (ret == 0) {
-			fprintf(stderr,"SSL_connect:failed in %s %s\n",
-				SSL_state_string(s),SSL_state_string_long(s));
-		} else if (ret < 0) {
-			fprintf(stderr,"SSL_connect:error in %s %s\n",
-				SSL_state_string(s),SSL_state_string_long(s));
-		}
-	}
-}
-
-#endif /* USE_SSL */
-
-
 #ifndef       MAXHOSTNAMELEN
 #define       MAXHOSTNAMELEN 64
 #endif /* MAXHOSTNAMELEN */
@@ -1465,6 +1422,10 @@ bye(argc, argv)
     extern int resettermname;
 
     if (connected) {
+#ifdef USE_SSL
+	if (ssl_active_flag)
+	  SSL_shutdown(ssl_con);
+#endif /* USE_SSL */
 	(void) shutdown(net, 2);
 	printf("Connection closed.\n");
 	(void) NetClose(net);
@@ -1591,7 +1552,7 @@ struct envlist {
 	int	narg;
 };
 
-extern struct env_lst *
+static struct env_lst *
 	env_define P((unsigned char *, unsigned char *));
 static void
 	env_undefine P((unsigned char *)),
@@ -2239,6 +2200,20 @@ tn(argc, argv)
 	}
 	if (hostp == 0) {
 	    hostp = *argv++;
+
+	    /* support host:port syntax that is now commonly
+	     * used in lots of programs --tjh 
+	     * (though don't disturb the existing IP_OPTIONS 
+	     *  code that has a colon in the syntax)
+	     */
+	    if ((hostp[0]!='@')&&(hostp[0]!='!')) {
+	      char *p;
+
+	      if ((p=strchr(hostp,':'))!=NULL) {
+	        *p='\0';
+		portp=p+1;
+	      }
+	    }
 	    --argc;
 	    continue;
 	}
@@ -2333,6 +2308,9 @@ tn(argc, argv)
 		    sin.sin_port = 443;
 		else if (strcmp(portp,"ssl-telnet")==0)
 		    sin.sin_port = 151;
+		/* include the offical IANA port! */
+		else if (strcmp(portp,"telnets")==0)
+		    sin.sin_port=992;
 		else
 #endif /* USE_SSL */
                 {
@@ -2421,73 +2399,26 @@ tn(argc, argv)
 	}
 
 #ifdef USE_SSL
-	if (ssl_debug_flag) {
-	    fprintf(stderr,"SSL_DEBUG_FLAG on\n");
+	if (!do_ssleay_init(0)) {
+	  if (bio_err==NULL) {
+	    fflush(stdout);
 	    fflush(stderr);
-	    /* take these two out of the "normal" stream */
-	    SSL_debug("telnet.log");
-	    /*
-	    (void)freopen("telnet.err","w",stderr);
-	    SSL_ERR=stderr;
-	    */
-	    SSL_LOG=SSL_ERR=stderr;
-	} else {
-	    SSL_LOG=SSL_ERR=NULL;
+	    fprintf(stderr,"do_ssleay_init() failed\n");
+	    ERR_print_errors_fp(stderr);
+	  } else {
+	    BIO_printf(bio_err,"do_ssleay_init() failed\n");
+	    ERR_print_errors(bio_err);
+	  }
+	  exit(1);
 	}
+	
+	/* bind in the network descriptor */
+    	SSL_set_fd(ssl_con,net);
 
-	/* init things so we will get meaningful error messages
-	 * rather than numbers 
-	 */
-	SSL_load_error_strings();
-
-	ssl_ctx=(SSL_CTX *)SSL_CTX_new();
-	ssl_con=(SSL *)SSL_new(ssl_ctx);
-
-        SSL_set_fd(ssl_con,net);
-        SSL_set_verify(ssl_con,ssl_verify_flag,NULL);
-	if (cipher_list==NULL)
-	    SSL_set_pref_cipher(ssl_con,getenv("SSL_CIPHER"));
-	else
-	    SSL_set_pref_cipher(ssl_con,cipher_list);
-
-	/* for verbose we use the 0.6.x info callback that I got
-	 * eric to finally add into the code :-) --tjh
-	 */
-	if (ssl_verbose_flag) {
-	    SSL_CTX_set_info_callback(ssl_ctx,client_info_callback);
-	}
-
-	/* Add in any certificates if you want to here ... */
-	if (cert_file) {
-	    if (!SSL_use_certificate_file(ssl_con, cert_file,
-		X509_FILETYPE_PEM)) {
-		fprintf(stderr,"%s: ",cert_file);
-		ERR_print_errors_fp(stderr);
-		exit(1);
-	    } else {
-		if (!key_file)
-		    key_file = cert_file;
-		if (!SSL_use_RSAPrivateKey_file(ssl_con, key_file,
-		    X509_FILETYPE_PEM)) {
-		    fprintf(stderr,"%s: ", key_file);
-		    ERR_print_errors_fp(stderr);
-		    exit(1);
-		}
-	    }
-	}
-
-	/* make sure we will find certificates in the standard
-	 * location ... otherwise we don't look anywhere for
-	 * these things which is going to make client certificate
-	 * exchange rather useless :-)
-	 */
-	SSL_set_default_verify_paths(ssl_ctx);
-
+	/* if we are doing raw SSL then start it now ... */
 	if (ssl_only_flag) {
-	    SSL_set_verify(ssl_con,ssl_verify_flag,
-					client_verify_callback);
 	    if (!SSL_connect(ssl_con)) {
-                static char errbuf[1024];
+		static char errbuf[1024];
 
 		ERR_print_errors_fp(stderr);
 		perror("SSL_connect");
@@ -2499,7 +2430,9 @@ tn(argc, argv)
 		ssl_active_flag=1;
 	    }
 	}
+
 #endif /* USE_SSL */
+
 	connected++;
 #if	defined(AUTHENTICATE) || defined(ENCRYPT)
 	auth_encrypt_connect(connected);
