@@ -176,13 +176,13 @@ main(argc, argv)
 			    ssl_verify_flag=atoi(optarg+strlen("verify="));
 			} else if (strncmp(optarg, "cert=", 
 			                        strlen("cert=")) == 0 ) {
-			    cert_file= optarg + strlen("cert=");
+			    ssl_cert_file= optarg + strlen("cert=");
 			} else if (strncmp(optarg, "key=", 
 			                        strlen("key=")) == 0 ) {
-			    key_file= optarg + strlen("key=");
+			    ssl_key_file= optarg + strlen("key=");
 			} else if (strncmp(optarg,"cipher=",
 			                strlen("cipher="))==0) {
-			    cipher_list=optarg+strlen("cipher=");
+			    ssl_cipher_list=optarg+strlen("cipher=");
 			} else if (strcmp(optarg, "socks") == 0 ) {
 #ifdef USE_SOCKS
 			    socks_flag=1;
