@@ -34,8 +34,9 @@
 /*
  * 28-Apr-95 tjh        IRIX3,IRIX4,IRIX5 support added
  *
- * tjh@mincom.oz.au
  * Tim Hudson
+ * tjh@cryptsoft.com
+ *
  */
 
 #ifndef lint
@@ -104,6 +105,19 @@ char	wtmpf[]	= "/etc/wtmp";
 # define EXTPROC 0400
 #endif
 
+#ifdef USE_SSL
+#define DO_SHUTDOWN(x,y) \
+			 if (debug) { \
+			   fprintf(stderr,"Doing SSL_shutdown\n");\
+			   fflush(stderr); \
+			 } \
+			 if (ssl_active_flag) \
+			   SSL_shutdown(ssl_con); \
+			 shutdown((x),(y))
+#else
+#define DO_SHUTDOWN(x,y) shutdown((x),(y))
+#endif
+
 #ifndef	USE_TERMIO
 struct termbuf {
 	struct sgttyb sg;
@@ -1541,19 +1555,24 @@ cleanup(sig)
 	*p = 'p';
 	(void)chmod(line, 0666);
 	(void)chown(line, 0, 0);
-	(void) shutdown(net, 2);
+	DO_SHUTDOWN(net, 2);
 	exit(1);
 # else
 	void rmut();
 
+	if (debug) {
+		fprintf(stderr,"cleanup(%d)\n",sig);
+		fflush(stderr);
+	}
+
 	rmut();
 	vhangup();	/* XXX */
-	(void) shutdown(net, 2);
+	DO_SHUTDOWN(net, 2);
 	exit(1);
 # endif
 #else	/* CRAY */
 # ifdef	NEWINIT
-	(void) shutdown(net, 2);
+	DO_SHUTDOWN(net, 2);
 	exit(1);
 # else	/* NEWINIT */
 	static int incleanup = 0;
@@ -1583,7 +1602,7 @@ cleanup(sig)
 	setutent();	/* just to make sure */
 	rmut(line);
 	close(pty);
-	(void) shutdown(net, 2);
+	DO_SHUTDOWN(net, 2);
 	if (t == 0)
 		cleantmp(&wtmp);
 	exit(1);
