@@ -1,6 +1,6 @@
 /* 
  * The modifications to support SSLeay were done by Tim Hudson
- * tjh@mincom.oz.au
+ * tjh@cryptsoft.com
  *
  * You can do whatever you like with these patches except pretend that
  * you wrote them. 
@@ -71,6 +71,10 @@ int	require_SecurID = 0;
 
 #include <sys/ioctl.h>
 
+#ifdef USE_SSL 
+static char cert_filepath[1024];
+#endif /* USE_SSL */
+
 
 /*
  * I/O data buffers,
@@ -92,32 +96,6 @@ char *progname;
 
 extern void usage P((void));
 
-#ifdef USE_SSL
-
-/*
-#include "ssl_err.h"
-*/
-
-SSL *ssl_con;
-SSL_CTX *ssl_ctx;
-int ssl_debug_flag=0;
-int ssl_only_flag=0;
-int ssl_active_flag=0;
-int ssl_secure_flag=0;
-int ssl_certsok_flag=0;
-int ssl_cert_required=0;
-int ssl_verify_flag=SSL_VERIFY_NONE;
-char ssl_file_path[1024];    /* don't look at that nasty value to the left */
-
-int ssl_verbose_flag=0;
-int ssl_disabled_flag=0;
-
-static char *my_ssl_key_file=NULL;
-static char *my_ssl_cert_file=NULL;
-char *cipher_list;
-
-#endif /* USE_SSL */
-
 main(argc, argv)
 	char *argv[];
 {
@@ -143,6 +121,17 @@ main(argc, argv)
 
 	progname = *argv;
 
+#ifdef USE_SSL
+	/* we need to know the fullpath to the location of the
+	 * certificate that we will be running with as we cannot
+	 * be sure of the cwd when we are launched
+	 */
+	sprintf(cert_filepath,"%s/%s",X509_get_default_cert_dir(),
+	        "telnetd.pem");
+	ssl_cert_file=cert_filepath;
+	ssl_key_file=NULL;
+#endif /* USE_SSL */
+
 #ifdef CRAY
 	/*
 	 * Get number of pty's before trying to process options,
@@ -185,13 +174,13 @@ main(argc, argv)
 			    ssl_verify_flag=atoi(optarg+strlen("verify="));
 			} else if (strncmp(optarg, "cert=", 
 			                strlen("cert=")) == 0 ) {
-			    my_ssl_cert_file=optarg+strlen("cert=");
+			    ssl_cert_file=optarg+strlen("cert=");
 			} else if (strncmp(optarg, "key=", 
 			                strlen("key=")) == 0 ) {
-			    my_ssl_key_file=optarg+strlen("key=");
+			    ssl_key_file=optarg+strlen("key=");
 			} else if (strncmp(optarg,"cipher=",
 			                strlen("cipher="))==0) {
-			    cipher_list=optarg+strlen("cipher=");
+			    ssl_cipher_list=optarg+strlen("cipher=");
 			} else {
 			    /* report when we are given rubbish so that
 			     * if the user makes a mistake they have to
@@ -389,57 +378,39 @@ main(argc, argv)
 	     * verify checking otherwise we cannot abort connections
 	     * at the right place!
 	     */
-	    if (ssl_verify_flag == 0)
-		ssl_verify_flag = 1;
+	    if (ssl_verify_flag==0)
+		ssl_verify_flag=1;
 	}
 
-        {
-	    int i;
-
-	    /* if we are not running in debug then any error
-	     * stuff from SSL debug *must* not go down
-	     * the socket (which 0,1,2 are all pointing to by
-	     * default)
-	     */
-	    if (ssl_debug_flag) {
-		if (!debug) {
-		    (void)freopen("/telnetd.err","w",stderr);
-		    SSL_ERR=stderr;
-		}
-		SSL_debug("/telnetd.log");
-
-		fprintf(stderr,"LOGGING to telnetd.err and telnetd.log\n");
-
-		fprintf(stderr,"secure %d certrequired %d verify %d\n",
-		    ssl_secure_flag,ssl_cert_required,ssl_verify_flag);
-		for(i=0;i<argc;i++)
-		    fprintf(stderr,"argv[%d]=\"%s\"\n",i,argv[i]);
-		fflush(stderr);
-
-	    } else {
-		/* disable all the debug and trace */
-		SSL_LOG=SSL_ERR=NULL;
-	    }
-
-	    SSL_load_error_strings();
-
-	    ssl_ctx=(SSL_CTX *)SSL_CTX_new();
-
-	    /* make sure we have access to the certificate area 
-	     * otherwise we will not be able to verify any client
-	     * certificates
-	     */
-            if (!SSL_set_default_verify_paths(ssl_ctx)) {
-		fprintf(stderr,"telnetd: cannot load certificates via SSL_set_default_verify_paths\n");
-		fflush(stderr);
-		ERR_print_errors_fp(stderr);
-		exit(1);
-	    }
+	/* if we are not running in debug then any error
+	 * stuff from SSL debug *must* not go down
+	 * the socket (which 0,1,2 are all pointing to by
+	 * default)
+	 */
+	if (ssl_debug_flag)
+	    ssl_log_file="/telnetd.log";
 
+	if (!do_ssleay_init(1)) {
+	  if (bio_err!=NULL) {
+	    BIO_printf(bio_err,"do_ssleay_init() failed\n");
+	    ERR_print_errors(bio_err);
+	  } else {
+	    fflush(stderr);
+	    fprintf(stderr,"do_ssleay_init() failed\n");
+	    ERR_print_errors_fp(stderr);
+	  }
+	  exit(1);
 	}
+
+	if (ssl_debug_flag) {
+	  BIO_printf(bio_err,"secure %d certrequired %d verify %d\n",
+	      ssl_secure_flag,ssl_cert_required,ssl_verify_flag);
+	  for(i=0;i<argc;i++)
+	      BIO_printf(bio_err,"argv[%d]=\"%s\"\n",i,argv[i]);
+	}
+
 #endif /* USE_SSL */
 
-
 	argc -= optind;
 	argv += optind;
 
@@ -541,42 +512,8 @@ main(argc, argv)
 
 #ifdef USE_SSL
         /* do the SSL stuff now ... before we play with pty's */
-	ssl_con=(SSL *)SSL_new(ssl_ctx);
 	SSL_set_fd(ssl_con,0);
 
-        /* just use telnetd.pem for public and private and don't
-	 * encrypt it with a pass phrase 
-	 */
-	sprintf(ssl_file_path,"%s/%s",X509_get_default_cert_dir(),
-	        "telnetd.pem");
-	if (SSL_use_RSAPrivateKey_file(ssl_con,
-	        my_ssl_key_file==NULL?ssl_file_path:my_ssl_key_file,
-		X509_FILETYPE_PEM)<=0) {
-	    fprintf(stderr,"SSL_use_RSAPrivateKey_file(%s) error ",
-	        my_ssl_key_file==NULL?ssl_file_path:my_ssl_key_file);
-	    ERR_print_errors_fp(stderr);
-	    fflush(stderr);
-	    sleep(5);
-	    _exit(1);
-	}
-
-	sprintf(ssl_file_path,"%s/%s",X509_get_default_cert_dir(),
-	        "telnetd.pem");
-	if (SSL_use_certificate_file(ssl_con,
-	        my_ssl_cert_file==NULL?ssl_file_path:my_ssl_cert_file,
-	        X509_FILETYPE_PEM)<=0) {
-	    fprintf(stderr,"SSL_use_certificate_file(%s) error ",
-	        my_ssl_cert_file==NULL?ssl_file_path:my_ssl_cert_file);
-	    ERR_print_errors_fp(stderr);
-	    fflush(stderr);
-	    sleep(5);
-	    _exit(1);
-	}
-
-	/* if we gave a cipher list then ... */
-        if (cipher_list!=NULL)
-	    SSL_set_pref_cipher(ssl_con,cipher_list);
-
 	if (ssl_only_flag) {
 	    /* hmm ... only when running talking to things like
 	     * https servers should we hit this code and then
@@ -592,17 +529,17 @@ main(argc, argv)
 
 		syslog(LOG_WARNING, errbuf);
 
-		fprintf(stderr,errbuf);
-		fflush(stderr);
+		BIO_printf(bio_err,errbuf);
 
+		/* go to sleep to make sure we are noticed */
 		sleep(10);
 		SSL_free(ssl_con);
+
 		_exit(1);
 	    } else {
 		ssl_active_flag=1;
 	    }
 	}
-
 #endif /* USE_SSL */
 
 	net = 0;
@@ -945,7 +882,7 @@ doit(who)
 	hp = gethostbyaddr((char *)&who->sin_addr, sizeof (struct in_addr),
 		who->sin_family);
 	if (hp)
-		host = hp->h_name;
+		host = (char *)hp->h_name;
 	else
 		host = inet_ntoa(who->sin_addr);
 	/*
@@ -1409,6 +1346,18 @@ telnet(f, p, host)
 		 */
 		if (FD_ISSET(p, &ibits)) {
 			pcc = read(p, ptyibuf, BUFSIZ);
+			if (debug) {
+				fprintf(stderr,"pty_read: %d %d\n",pcc,errno);
+				fflush(stderr);
+			}
+#ifdef LINUX
+			/* EIO on pty means closed under Linux 2.x kernels
+			 * at least ... so we clear it so it isn't treated
+			 * as EWOULDBLOCK by the code below! --tjh
+			 */
+			if (errno==EIO)
+				errno=0;
+#endif /* LINUX */
 			/*
 			 * On some systems, if we try to read something
 			 * off the master side before the slave side is
